/**
 * (C) 2011-2012 Alibaba Group Holding Limited.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 */
package com.taobao.common.tedis.core;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import com.taobao.common.tedis.binary.DataType;
import com.taobao.common.tedis.serializer.TedisSerializer;
import com.taobao.common.tedis.util.SortParams;

/**
 * ڣ
 * <ul>
 *  <li>ĻkeyĲ</li>
 *  <li>ȡݽṹ</li>
 *  <li></li>
 *  <li>ȡõǰлʽ</li>
 * </ul>
 * @author juxin.zj E-mail:juxin.zj@taobao.com
 * @since 2011-7-25 11:54:51
 * @version 1.0
 */
public interface TedisManager {

    /**
     * <tt>key</tt>Ƿ
     * @param namespace
     * @param key
     * @return
     */
    <K> Boolean hasKey(int namespace, K key);

    /**
     * ɾָ<tt>key</tt>
     * @param namespace
     * @param key
     */
    <K> void delete(int namespace, K key);

    /**
     * ɾ<tt>key</tt>
     * @param namespace
     * @param keys
     */
    <K> void delete(int namespace, Collection<K> keys);

    /**
     * ȡ<tt>key</tt>
     * @param namespace
     * @param key
     * @return
     */
    <K> DataType type(int namespace, K key);

    /**
     * ȡ<tt>pattern</tt>ƥ<tt>key</tt>
     * @param namespace
     * @param pattern
     * @return
     */
    <K> Set<K> keys(int namespace, String pattern);

    /**
     * <tt>key</tt>
     * @param namespace
     * @param oldKey
     * @param newKey
     */
    <K> void rename(int namespace, K oldKey, K newKey);

    /**
     * <tt>oldKey</tt><tt>oldKey</tt>Ϊ<tt>newKey</tt>
     * @param namespace
     * @param oldKey
     * @param newKey
     * @return
     */
    <K> Boolean renameIfAbsent(int namespace, K oldKey, K newKey);

    /**
     * <tt>key</tt>Ĺʱ
     * @param namespace
     * @param key
     * @param timeout
     * @param unit
     * @return
     */
    <K> Boolean expire(int namespace, K key, long timeout, TimeUnit unit);

    /**<tt>key</tt>Ĺʱ
     * @param namespace
     * @param key
     * @param date
     * @return
     */
    <K> Boolean expireAt(int namespace, K key, Date date);

    /**
     * ȡ<tt>key</tt>Ĺ
     * @param namespace
     * @param key
     * @return
     */
    <K> Boolean persist(int namespace, K key);

    /**
     * ȡ<tt>key</tt>Ĺʱ
     * @param namespace
     * @param key
     * @return
     */
    <K> Long getExpire(int namespace, K key);

    /**
     * ȡü
     * @see AtomicCommands
     * @return
     */
    AtomicCommands getAtomicCommands();

    /**
     * ȡString
     * @see StringCommands
     * @return
     */
    StringCommands getStringCommands();

    /**
     * ȡһkey value
     * @see ValueCommands
     * @return
     */
    ValueCommands getValueCommands();

    /**
     * ȡHash
     * @see HashCommands
     * @param <HK>
     * @param <HV>
     * @return
     */
    HashCommands getHashCommands();

    /**
     * ȡList
     * @see ListCommands
     * @return
     */
    ListCommands getListCommands();

    /**
     * ȡSet
     * @see SetCommands
     * @return
     */
    SetCommands getSetCommands();

    /**
     * ȡSet
     * @see ZSetCommands
     * @return
     */
    ZSetCommands getZSetCommands();

    /**
     * 
     * @param namespace
     * @param query
     * @return
     */
    <K, V> List<V> sort(int namespace, K key, SortParams params);

    /**
     * 
     * @param namespace
     * @param query
     * @param storeKey
     * @return
     */
    <K> Long sort(int namespace, K key, SortParams query, K storeKey);

    /**
     * ȡ<tt>value</tt>л
     * @return
     */
    TedisSerializer<?> getValueSerializer();

    /**
     * ȡ<tt>key</tt>л
     * @return
     */
    TedisSerializer<?> getKeySerializer();

    /**
     * Դֹ̳ͣ߳
     */
    void destroy();

}
